/*
 * T.C.F.S. 2.0 Alpha 1 
 *
 *      	   This  program  handles  RPC  "NFS"  data  requests
 *              adopting a secure transfer protocol.
 *                 This   is  an  unsecure   and  unchecked  version,
 *              use at your own risk.
 *
 *              Please, report Bugs to: <tcfs@edu-gw.dia.unisa.it>
 *
 * Authors:	Giuseppe Cattaneo, <cattaneo@udsab.dia.unisa.it>
 *		Giuseppe Persiano, <giuper@udsab.dia.unisa.it>
 *		Andrea Cozzolino, <andcoz@edu-gw.dia.unisa.it>
 *		Angelo Celentano, <angcel@edu-gw.dia.unisa.it>
 *		Aniello Del Sorbo, <anidel@edu-gw.dia.unisa.it>
 *		Ermelindo Mauriello, <ermmau@edu-gw.dia.unisa.it>
 *		Raffaele Pisapia, <rafpis@edu-gw.dia.unisa.it>
 *
 *   Permission to  use, copy, and modify  this software  without fee
 * is hereby granted, provided that this entire notice is included in
 * all copies  of  any  software  which  is  or  includes a  copy  or
 * modification of this software and in all copies  of the supporting
 * documentation for such software.
 *
 *   This  software is  distribuited  under  the  GNU General  Public
 * License  (version  2, June  1991). Check  the  file  'COPYING'  for
 * more  infos. Some  parts of  this  software  derive  from the  NFS
 * implementation in the Linux kernel 2.0.x.
 *
 * This software  maybe be used  for any  purpose provided  the above
 * copyright  notice  is retained.  It  is  supplied  as is,  with no
 * warranty expressed or implied.
 *
 */

/* -+-_== */

#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/fcntl.h>
#include <linux/stat.h>
#include <linux/mm.h>
#include <linux/tcfs_fs.h>
#include <linux/malloc.h>
#include <linux/pagemap.h>

#include <asm/segment.h>
#include <asm/system.h>
#include <linux/autoconf.h>

#ifdef TCFS_DES

#include <linux/kdes.h>

void ides(char *input,int numblocks,des_key_schedule *ks,int mode)
{
	static int i;
/*	des_key_schedule ks;*/
	char in[8],*tmp;
	tmp=input;
/*	des_set_key((des_cblock *)key,ks);*/
#ifdef DEBUG_TCFS
/*	printk("Dopo set_key %d mode %d\n",numblocks,mode); */
#endif
	for (i=0;i<numblocks;i++)
	{
		memcpy_fromfs(in,(char *)tmp,8L);
		des_ecb_encrypt((des_cblock *)in,(des_cblock *)in,*ks,mode);
		memcpy_tofs((char *)tmp,in,8L);
		tmp+=8;
	}
}      

void ikdes(char *input,int nb,des_key_schedule *ks,int mode)
{
	static int i;
	char *tmp;
	tmp=input;
	for (i=0;i<nb;i+=8)
	{
		des_ecb_encrypt((des_cblock *)tmp,(des_cblock *)tmp,*ks,mode);
		tmp+=8;
	}
}      

#elif TCFS_IDEA

#include <linux/idea.h>

void encrypt(char *input,int numblocks,idea_key_schedule *ks)
{
	static int i;
	char in[8],*tmp;
	tmp=input;

	for (i=0;i<numblocks;i++)
	{
		memcpy_fromfs(in,(char *)tmp,8L);
		idea_encrypt(in,ks[0]);
		memcpy_tofs((char *)tmp,in,8L);
		tmp+=8;
	}
}      

void decrypt(char *input,int numblocks,idea_key_schedule *ks)
{
	static int i;
	char in[8],*tmp;
	tmp=input;

	for (i=0;i<numblocks;i++)
	{
		memcpy_fromfs(in,(char *)tmp,8L);
		idea_decrypt(in,ks[1]);
		memcpy_tofs((char *)tmp,in,8L);
		tmp+=8;
	}
}      

void kencrypt(char *input,int nb,idea_key_schedule *ks)
{
	static int i;
	char *tmp;
	tmp=input;
	for (i=0;i<nb;i+=8)
	{
		idea_encrypt(tmp,ks[0]);
		tmp+=8;
	}
}      

void kdecrypt(char *input,int nb,idea_key_schedule *ks)
{
	static int i;
	char *tmp;
	tmp=input;
	for (i=0;i<nb;i+=8)
	{
		idea_decrypt(tmp,ks[1]);
		tmp+=8;
	}
}      

#elif TCFS_RC5

#include <linux/rc5.h>

void encrypt(char *input,int numblocks,rc5_key_schedule *ks)
{
	static int i;
	char in[8],*tmp;
	WORD ct[2];
	tmp=input;

	for (i=0;i<numblocks;i++)
	{
		memcpy_fromfs(in,(char *)tmp,8L);
		rc5_encrypt(in,ct,(WORD *)ks);
		memcpy_tofs((char *)tmp,ct,8L);
		tmp+=8;
	}
}      

void decrypt(char *input,int numblocks,rc5_key_schedule *ks)
{
	static int i;
	char in[8],*tmp;
	WORD ct[2];
	tmp=input;

	for (i=0;i<numblocks;i++)
	{
		memcpy_fromfs(in,(char *)tmp,8L);
		rc5_decrypt(in,ct,(WORD *)ks);
		memcpy_tofs((char *)tmp,ct,8L);
		tmp+=8;
	}
}      

void kencrypt(char *input,int nb,rc5_key_schedule *ks)
{
	static int i;
	char *tmp;
	char ct[8];
	tmp=input;
	for (i=0;i<nb;i+=8)
	{
		rc5_encrypt(tmp,(WORD *)ct,(WORD *)ks);
		memcpy((char *)tmp,ct,8L);
		tmp+=8;
	}
}      

void kdecrypt(char *input,int nb,rc5_key_schedule *ks)
{
	static int i;
	char *tmp;
	char ct[8];
	tmp=input;
	for (i=0;i<nb;i+=8)
	{
		rc5_decrypt(tmp,(WORD *)ct,(WORD *)ks);
		memcpy((char *)tmp,ct,8L);
		tmp+=8;
	}
}      

#endif